open YaccTypes;;
open HeadFinder;;

let emptyNode = {word="";cat="";pos=(-1);parent=(-1);mainChild=(-1);edge=""};;

let rec getNode nl fl i = match nl with
    [] -> (match (List.length fl) with
		        0 -> emptyNode
			     |_ -> (List.nth fl ((List.length fl) -1)))
  |h::t -> if (h.pos == i) then h else (getNode t fl i);;

(*let rec getNode nl fl i = match nl with
    [] -> (List.nth fl ((List.length fl) -1))
  |h::t -> if (h.pos == i) then h else (getNode t fl i);;*)

let rec separateNodes nl wl pl = match nl with
    [] -> (List.rev pl), (List.rev wl)
  |h::t -> if (h.pos > 499) then (separateNodes t wl (h::pl)) else (separateNodes t (h::wl) pl);;

let rec makeMain n parentL = match parentL with
    [] -> []
  |h::t -> if (n.parent == h.pos) then [{word=h.word;cat=h.cat;pos=h.pos;parent=h.parent;mainChild=n.pos;edge=h.edge}] else (makeMain n t);;

let rec findParentWithHead i l fl = match l with
    [] -> (match (List.length fl) with 
0 -> (emptyNode)
      |_ -> (List.hd fl))
  |h::t -> match (i == h.pos) with
true -> (match (h.mainChild != (-1)) with
    true -> h 
  |false -> (findParentWithHead h.parent fl fl))
       |false -> (findParentWithHead i t fl);;

(*let rec findParentWithHead i l fl = match l with
    [] -> (List.hd fl)
  |h::t -> if (i == h.pos) then (if (h.mainChild != (-1)) then h else (findParentWithHead h.parent fl fl)) else (findParentWithHead i t fl);;*)

let rec matchNodes a nodes x = match nodes with
    [] -> []
  |h::t -> match (h.parent = x.pos) with
	false -> (matchNodes a t x)
      |true -> if ((compare a h.cat) == 0) then [h] else (matchNodes a t x);;

let rec getHead heads nodes parents x = match heads with
    [] -> (findParentWithHead x.parent parents parents).mainChild
  |h::t -> match (matchNodes h nodes x) with
	[] -> (getHead t nodes parents x)
      |mn::t -> mn.pos;;

let rec findRule x rules nodes parents = match rules with
    [] -> (findParentWithHead x.parent parents parents).mainChild
  |h::t -> 
     if ((compare h.name x.cat) == 0) 
     then 
       (if (h.dir == Left) 
        then (getHead h.heads (List.rev nodes) parents x) 
        else (getHead h.heads nodes parents x)) 
     else (findRule x t nodes parents);;

let rec headFind parentL headRules nodeL newParentL = match parentL with
    [] -> newParentL
  |h::t -> match h.mainChild with
	-1 -> let head = (findRule h headRules nodeL newParentL) in
		(headFind t headRules nodeL ({word=h.word;cat=h.cat;pos=h.pos;parent=h.parent;mainChild=head;edge=h.edge}::newParentL))
      |x -> (headFind t headRules nodeL newParentL);;

let rec getMainKid parentL nodeL fullNL newL = match nodeL with
    [] -> (headFind (List.rev parentL) headRules fullNL newL)
  |h::t -> match h.edge with
	"HD" -> getMainKid parentL t fullNL (newL@(makeMain h parentL))
      |_ -> (getMainKid parentL t fullNL newL);;


let rec findParent n pl =
  let parentNode = (getNode pl pl n.parent) in
  let grandParentNode = (getNode pl pl parentNode.parent) in 
  let head = parentNode.mainChild in
    match (n.pos != head) with
	true -> {word=n.word;cat=n.cat;pos=n.pos;parent=head;mainChild=n.mainChild;edge=n.edge}
      |false -> match (parentNode.parent == 0) with
	    true -> {word=n.word; cat=n.cat;pos=n.pos;parent=(-1);mainChild=n.mainChild;edge=n.edge}
	  |false -> ({word=n.word; cat=n.cat; pos=n.pos; parent=grandParentNode.mainChild; mainChild=n.mainChild;edge=n.edge});;


let iterNodes parentL nodeL = 
  let pl = (getMainKid parentL nodeL nodeL []) in
  let rec itNo parL nodeL newL = match nodeL with
     [] -> (List.rev newL)
   |h::t -> (itNo parL t ((findParent h parL)::newL))
  in (itNo pl nodeL []);;

let rec iterSentences sentences newL = match sentences with
    [] -> (List.rev newL)
  |h::t -> let pl, nl = (separateNodes h [] []) in
     match nl with
 [] -> (iterSentences t newL)
       |hh::tt -> (iterSentences t ((iterNodes pl nl)::newL));;



(*let dummySentence =
  [[{word="Den"; cat="ART"; pos=(0); parent=500; mainChild=(-1); edge="NK"};
{word="Ton"; cat="NN"; pos=(1); parent=500; mainChild=(-1); edge="NK"};
{word="gab"; cat="VVFIN"; pos=(2); parent=503; mainChild=(-1); edge="HD"};
{word="der"; cat="ART"; pos=(3); parent=501; mainChild=(-1); edge="NK"};
{word="Kuenstler"; cat="NN"; pos=(4); parent=501; mainChild=(-1); edge="NK"};
{word="seinem"; cat="PPOSAT"; pos=(5); parent=502; mainChild=(-1); edge="NK"};
{word="Gehilfen"; cat="NN"; pos=(6); parent=502; mainChild=(-1); edge="NK"};
{word="."; cat="$."; pos=(7); parent=0; mainChild=(-1); edge=""};
{word="#500"; cat="NP"; pos=(500); parent=503; mainChild=(-1); edge="OA"};
{word="#501"; cat="NP"; pos=(501); parent=503; mainChild=(-1); edge="SB"};
{word="#502"; cat="NP"; pos=(502); parent=503; mainChild=(-1); edge="DA"};
{word="#503"; cat="NP"; pos=(503); parent=0; mainChild=(-1); edge=""}]];;
*)



