open YaccTypes;;
open Nivre;;
open DepToTrans;;

(*UTILITY FUNCTIONS*)

let configOut = (open_out "../Output/configs.txt");;

let string_of_op x = match x with
    Left -> "LEFT"
  |Right -> "RIGHT"
  |Shift -> "SHIFT"
  |Reduce -> "REDUCE";;

(*PRINTERS*)
let rec getSentence wordList s = match wordList with
    [] -> s
  |h::t -> (getSentence t  (s^h.word^"|"^h.cat^"|"^(string_of_int h.pos)^" "));;

let rec printH a i = match (i < Array.length a) with
    false -> (output_string configOut "\t"); (flush configOut)
  |true -> match ((Array.get a i) == (-2)) with
	true -> (printH a (i+1))
      |false -> (output_string configOut ((string_of_int i)^"|"^(string_of_int (Array.get a i))^";")); (printH a (i+1));;
      
let rec printD a i = match (i < Array.length a) with
    false -> (flush configOut)
  |true -> match ((compare (Array.get a i) "") == 0) with
	true -> (printD a (i+1))
      |false -> (output_string configOut ((string_of_int i)^"|"^ (Array.get a i)^";")); (printD a (i+1));;

let rec printNodes n = match n with
    [] -> (output_string configOut "\t"); (flush configOut)
  |h::t -> ((output_string configOut (h.word^"|"^h.cat^"|"^(string_of_int h.pos)^"+")); printNodes t)

let printTrans t = (output_string configOut ("\t"^(string_of_op t.op)^"\t"^(string_of_int t.fromWord.pos)^"\t"^(string_of_int t.toWord.pos)^"\n")); flush configOut;;

let printConfig c = (printNodes c.stack); (printNodes c.tau); (printH c.h 0); (printD c.d 0); (output_string configOut "\n");;
let printConfigTrans c t = (printNodes c.stack); (printNodes c.tau); (printH c.h 0); (printD c.d 0); (printTrans t);;

let rec iterSent l = match l with
    [] -> (flush configOut)
  |h::t -> (printConfigTrans (fst h) (snd h)); (iterSent t);;

let rec printAll configHash i len = match (i < len) with
    false -> (flush configOut)
  |true -> 
     let item = (Hashtbl.find configHash i) in
       (output_string configOut ((fst item)^"\n")); (iterSent (snd item));(output_string configOut "\n");(printAll configHash (i+1) len);;

(*TransToConfig*)

let initialConfig sentenceList = {stack=[{word="ROOT";cat="ROOT";pos=(-1);parent=(-2);mainChild=(-1);edge=""}];
				  tau=sentenceList;
				  h=(Array.make (List.length sentenceList) (-2));
				  d=(Array.make (List.length sentenceList) "");
				  soFar=[]};;

let rec makeLeft t c l= match ((List.length c.stack) > 0) with
    false ->  c, l
  |true ->  match (t.toWord.pos == ((List.hd c.stack).pos)) with 
	true ->  (leftArc c t.toWord.pos), (c,t)::l
      |false -> (makeLeft t (reduce c) ((c,{op=Reduce;fromWord=(List.hd c.stack);toWord=(List.hd c.stack)})::l));;

let rec makeRight t c l= match ((List.length c.stack) > 0) with
    false -> c, l
  |true -> match (t.fromWord.pos == ((List.hd c.stack).pos)) with
	true ->  (rightArc c t.fromWord.pos), (c,t)::l
      |false -> (makeRight t (reduce c) ((c,{op=Reduce;fromWord=(List.hd c.stack);toWord=(List.hd c.stack)})::l));;


let makeShift t c l = (shift c), (c,t)::l;;

let rec matchTransition t c l= match t.op with
    Left -> makeLeft t c l
  |Right -> makeRight t c l
  |Shift -> makeShift t c l
  |_ -> c, l;;

let rec reduceLast config l = match config.stack with
    [] ->  l
  |h::t ->(reduceLast (reduce config) ((config,{op=Reduce;fromWord=(List.hd config.stack);toWord=(List.hd config.stack)})::l));;

let rec getConfig tList config l = match tList with
    [] -> (List.rev (reduceLast config l))
  |h::t -> let newC, newL = (matchTransition h config l) in
	     (getConfig t newC newL);;

let printMod i = match (i mod 100) with
    0 -> (match (i mod 500) with
	0 -> (match (i mod 1000) with
	    0 -> (print_endline ("*"^(string_of_int i)^" sentences converted."))
	  |_ ->print_char '*')
       |_ -> print_char '.')
   |_ -> (flush stdout);;

(*let rec run transitionHash sentenceList i len configHash = match i with
  130000 -> configHash
  |_ -> match (i < len) with
	 false -> configHash
       |true -> (Hashtbl. add configHash i 
		    ((getSentence (Hashtbl.find sentenceHash i) ""),(getConfig (Hashtbl.find transitionHash i) (initialConfig (Hashtbl.find sentenceHash i)) [])));
                    (printMod i);
	            (run transitionHash sentenceHash (i+1) len configHash);;

let configHash = (run transitionHash sentenceList 0 len (Hashtbl.create 130000));;
*)

let rec runT2C transitionHash sentenceList i configHash = match sentenceList with
    [] -> configHash
  |h::t -> let tElement = (try (Hashtbl.find transitionHash i) with 
                Invalid_argument "index" -> []
               |Not_found -> []) in
           match tElement with 
           [] -> runT2C transitionHash t (i+1) configHash
          |hh::tt -> 
      (Hashtbl.add configHash i
     ((getSentence h ""), (getConfig tElement (initialConfig h) []));
     (printMod i);
     (runT2C transitionHash t (i+1) configHash));;
